
local _G = _G
local pairs = pairs
local sfind = string.find
local tconcat = table.concat
local tinsert = table.insert
local tsort = table.sort
local tremove = table.remove

-- Frames
local GsFrame
local btnClose
local btnReloadUi
local btnRefresh
local fsSlot = {}
local fsIlvl = {}
local fsName = {}
local fsRegion = {}

-- Other
local itemtable = {}
local slots = {["Back"] = "BackSlot", ["Chest"] = "ChestSlot", ["Feet"] = "FeetSlot", ["Finger 1"] = "Finger0Slot", ["Finger 2"] = "Finger1Slot", ["Hands"] = "HandsSlot", ["Head"] = "HeadSlot", ["Legs"] = "LegsSlot", ["Main Hand"] = "MainHandSlot", ["Neck"] = "NeckSlot", ["Off Hand"] = "SecondaryHandSlot", ["Shoulders"] = "ShoulderSlot", ["Trinket 1"] = "Trinket0Slot", ["Trinket 2"] = "Trinket1Slot", ["Waist"] = "WaistSlot", ["Wrist"] = "WristSlot"}
local backdropTable = { bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = false, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 } }
BINDING_HEADER_GEAR_SUMMARY = "Gear Summary"
BINDING_NAME_GEAR_SUMMARY_TOGGLE = "Toggle Gear Summary Window"
BINDING_NAME_GEAR_SUMMARY_REFRESH = "Refresh Gear Summary Data"
SLASH_GEARSUMMARY1 = "/gs"

local function GetSlotStuffOld(slotName)
	local itemid = GetInventoryItemID("player",GetInventorySlotInfo(slots[slotName]))
	local t = {}
	if itemid ~= nil then
		t.slot = slots[slotName]
		t.slotName = slotName
		t.name,t.link,t.quality,t.ilvl,t.reqLevel,t.class,t.subclass,t.maxStack,t.equipSlot,t.texture,t.vendorPrice = GetItemInfo(itemid)
	else
		t = nil
	end
	return t
end

local function GetSlotStuff(slotName)
	local itemlink = GetInventoryItemLink("player",GetInventorySlotInfo(slots[slotName]))
	local t = {}
	if itemlink ~= nil then
		t.slot = slots[slotName]
		t.slotName = slotName
		t.name,t.link,t.quality,t.ilvl,t.reqLevel,t.class,t.subclass,t.maxStack,t.equipSlot,t.texture,t.vendorPrice = GetItemInfo(itemlink)
	else
		t = nil
	end
	return t
end

local function ReadItems()
	local i = 0
	for k, v in pairs(slots) do
		i = i + 1
		itemtable[i] = GetSlotStuff(k)
	end
	tsort(itemtable,
		function (a,b)
			if a == nil then
				return false
			elseif b == nil then
				return true
			elseif a.ilvl < b.ilvl then
				return true
			else
				return false
			end
		end
	)
end

local function RefreshData()
	itemtable = wipe(itemtable)
	ReadItems()
	local i = 0
	local maxWidth = 0
	for k, v in pairs(itemtable) do
		i = i + 1
		if v ~= nil then
			fsSlot[i]:SetText(v.slotName)
			fsIlvl[i]:SetText(v.ilvl)
			
			local tmpFontString = GsFrame:CreateFontString(nil, "OVERLAY")
			tmpFontString:SetWidth(600)
			tmpFontString:SetHeight(20)
			tmpFontString:SetFont("Fonts\\ARIALN.TTF", 14, "")
			tmpFontString:SetJustifyH("LEFT")
			tmpFontString:SetText(v.link)
			local thisWidth = tmpFontString:GetStringWidth()
			if thisWidth > maxWidth then
				maxWidth = thisWidth
			end	
			
			fsName[i]:SetText(v.link)
			fsName[i]:SetScript("OnHyperlinkEnter", function() GameTooltip:SetOwner(fsName[i], "ANCHOR_CURSOR"); GameTooltip:SetHyperlink(v.link); end)
			fsName[i]:SetScript("OnHyperlinkLeave", function() GameTooltip:Hide(); end)
		else
			fsSlot[i]:SetText("")
			fsIlvl[i]:SetText("")
			fsName[i]:SetText("")
			fsName[i]:SetScript("OnHyperlinkEnter", nil)
			fsName[i]:SetScript("OnHyperlinkLeave", nil)
		end
	end
	GsFrame:SetHeight(38+(i*20))
	while i < 16 do
		i = i + 1
		fsSlot[i]:SetText("")
		fsIlvl[i]:SetText("")
		fsName[i]:SetText("")
		fsName[i]:SetScript("OnHyperlinkEnter", nil)
		fsName[i]:SetScript("OnHyperlinkLeave", nil)
	end
	GsFrame:SetWidth(maxWidth + 162)
end

local function OnEvent(self, event, ...)
	if event == "UNIT_INVENTORY_CHANGED" then
		if GsFrame:IsShown() then
			RefreshData()
		end
	end
end

local function InitFrames()
	GsFrame = CreateFrame("Frame", "GsFrame", UIParent)
	tinsert(UISpecialFrames, "GsFrame")
	GsFrame:SetWidth(600)
	GsFrame:SetHeight(375)
	GsFrame:SetPoint("CENTER", UIParent, "CENTER")
	GsFrame:SetFrameStrata("DIALOG")
	GsFrame:SetBackdrop(backdropTable)
	GsFrame:SetBackdropColor(0, 0, 0, 1)
	GsFrame:SetMovable(true)
	GsFrame:SetClampedToScreen(true)
	GsFrame:EnableMouse(true)
	GsFrame:RegisterForDrag("LeftButton")
	GsFrame:SetScript("OnDragStart", function() GsFrame:StartMoving() end)
	GsFrame:SetScript("OnDragStop", function() GsFrame:StopMovingOrSizing() end)
	GsFrame:SetScript("OnEvent", OnEvent)
	GsFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")

	btnClose = CreateFrame("Button", nil, GsFrame, "UIPanelCloseButton")
	btnClose:SetPoint("TOPRIGHT", GsFrame, "TOPRIGHT")
	
	btnReloadUi = CreateFrame("Button", "btnReloadUi", GsFrame,"UIPanelButtonTemplate")
	btnReloadUi:SetText("RUI")
	btnReloadUi:SetPoint("TOPRIGHT",btnClose,"TOPLEFT",0,-5)
	btnReloadUi:Show()
	btnReloadUi:SetScript("OnClick", function() ReloadUI(); end)
	
	btnRefresh = CreateFrame("Button", "btnRefresh", GsFrame,"UIPanelButtonTemplate")
	btnRefresh:SetText("Refresh")
	btnRefresh:SetSize(60,20)
	btnRefresh:SetPoint("TOPRIGHT",btnReloadUi,"TOPLEFT",0,-1)
	btnRefresh:Show()
	btnRefresh:SetScript("OnClick", function() RefreshData(); end)
			
	local i = 0
	local maxWidth = 0
	for k, v in pairs(itemtable) do
		i = i + 1
		fsSlot[i] = GsFrame:CreateFontString(nil, "OVERLAY")
		fsSlot[i]:SetTextColor(1, 1, 1, 1)
		fsSlot[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 12, (-i*20)-10)
		fsSlot[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		fsSlot[i]:SetText(v.slotName)
		fsIlvl[i] = GsFrame:CreateFontString(nil, "OVERLAY")
		fsIlvl[i]:SetTextColor(1, 1, 1, 1)
		fsIlvl[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 100, (-i*20)-10)
		fsIlvl[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		fsIlvl[i]:SetText(v.ilvl)

		local tmpFontString = GsFrame:CreateFontString(nil, "OVERLAY")
		tmpFontString:SetWidth(600)
		tmpFontString:SetHeight(20)
		tmpFontString:SetFont("Fonts\\ARIALN.TTF", 14, "")
		tmpFontString:SetJustifyH("LEFT")
		tmpFontString:SetText(v.link)
		local thisWidth = tmpFontString:GetStringWidth()
		if thisWidth > maxWidth then
			maxWidth = thisWidth
		end	
		
		fsName[i] = CreateFrame("SimpleHTML","fsName"..i,GsFrame)
		fsName[i]:SetWidth(400)
		fsName[i]:SetHeight(1)
		fsName[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 150, (-i*20)-10)
		fsName[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		fsName[i]:SetText(v.link)
		fsName[i]:SetScript("OnHyperlinkEnter", function() GameTooltip:SetOwner(fsName[i], "ANCHOR_CURSOR"); GameTooltip:SetHyperlink(v.link); end)
		fsName[i]:SetScript("OnHyperlinkLeave", function() GameTooltip:Hide(); end)
	end
	GsFrame:SetHeight(38+(i*20))
	while i < 16 do
		i = i + 1
		fsSlot[i] = GsFrame:CreateFontString(nil, "OVERLAY")
		fsSlot[i]:SetTextColor(1, 1, 1, 1)
		fsSlot[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 12, (-i*20)-10)
		fsSlot[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		-- fsSlot[i]:SetText("")
		fsIlvl[i] = GsFrame:CreateFontString(nil, "OVERLAY")
		fsIlvl[i]:SetTextColor(1, 1, 1, 1)
		fsIlvl[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 100, (-i*20)-10)
		fsIlvl[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		-- fsIlvl[i]:SetText("")

		fsName[i] = CreateFrame("SimpleHTML","fsName"..i,GsFrame)
		fsName[i]:SetWidth(400)
		fsName[i]:SetHeight(1)
		fsName[i]:SetPoint("TOPLEFT", GsFrame, "TOPLEFT", 150, (-i*20)-10)
		fsName[i]:SetFont("Fonts\\ARIALN.TTF", 14, "")
		-- fsName[i]:SetText("")
		-- fsName[i]:SetScript("OnHyperlinkEnter", nil)
		-- fsName[i]:SetScript("OnHyperlinkLeave", nil)
	end
	GsFrame:SetWidth(maxWidth + 162)
end

local function SlashHandler(msg, editbox)
	if not GsFrame then		
		ReadItems()
		InitFrames()
	end
	GsFrame:Show()
end
SlashCmdList["GEARSUMMARY"] = SlashHandler;

function GsToggle()
	if not GsFrame then		
		ReadItems()
		InitFrames()
		GsFrame:Show()
	elseif GsFrame:IsShown() then
		GsFrame:Hide()
	else
		GsFrame:Show()
		RefreshData()
	end
end

function GsRefresh()
	RefreshData()
end
